import {
  Controller,
  Post,
  UseGuards,
  UsePipes,
  Param,
  Body,
  Get,
  Put,
  Delete,
} from '@nestjs/common';

import { AuthGuard } from '../shared/auth.guard';
import { ValidationPipe } from '../shared/validation.pipe';
import { User } from '../user/user.decorator';
import {
  GameGroupDTO,
  PromotePlayerDTO,
  JoinFactionDTO,
  JoinGameGroupDTO,
  FactionDTO,
} from './faction.dto';
import { FactionService } from './faction.service';
import { Roles, GameStates } from '../shared/guard.decorator';
import { GamePerson } from '../game/gameperson.decorator';

/////////////////////////////////////////////////////////////////////////////////
/// FactionController is being used for routing:                              ///
///                                                                           ///
/// Group                                                                     ///
/// - create group when game status is CREATED                                ///
/// - getting groups with faction id(this is used mainly for listing players) ///
/// - joining group when game status is CREATED                               ///
///                                                                           ///
/// Faction                                                                   ///
/// - checking users faction                                                  ///
/// - joining faction                                                         ///
/// - leaving faction                                                         ///
/// - changing faction multiplier (not implemented)                           ///
///                                                                           ///
/// See shared folder files for more information on decorators.               ///
/////////////////////////////////////////////////////////////////////////////////

@Controller('faction')
export class FactionController {
  constructor(private factionservice: FactionService) {}

  // takes gameId from the url to verify user role
  @Post('create-group/:id')
  @Roles('soldier')
  @GameStates('CREATED')
  @UsePipes(new ValidationPipe())
  async createGroup(
    @User('id') person,
    @Param('id') id: string,
    @Body() data: GameGroupDTO,
  ) {
    return this.factionservice.createGroup(person, id, data);
  }

  // id is faction ID
  @Get('get-groups/:id')
  async getGroups(@Param('id') id) {
    return this.factionservice.showGroups(id);
  }

  // takes gameId from the url to verify user role
  @Put('join-group/:id')
  @Roles('soldier')
  @GameStates('CREATED')
  async joinGroup(
    @GamePerson() gameperson,
    @Param('id') gameId,
    @Body() data: JoinGameGroupDTO,
  ) {
    return this.factionservice.joinGroup(gameperson, data);
  }

  // param game ID is passed to @Roles
  @Put('promote/:id')
  @Roles('admin')
  @GameStates('CREATED')
  @UsePipes(new ValidationPipe())
  promotePlayer(@Param('id') game, @Body() body: PromotePlayerDTO) {
    return this.factionservice.promotePlayer(body);
  }

  // used to join a faction
  // :id is the id of the game, and is needed for GameStates to check the state of the game
  @Put('join-faction/:id')
  @UseGuards(new AuthGuard())
  @GameStates('CREATED', 'STARTED', 'PAUSED')
  @UsePipes(new ValidationPipe())
  joinFaction(
    @User('id') person,
    @Param('id') game,
    @Body() data: JoinFactionDTO,
  ) {
    return this.factionservice.joinFaction(person, data);
  }

  // used to leave a faction
  // :id is the if of the game
  @Delete('leave/:id')
  @Roles('soldier', 'factionleader')
  @GameStates('CREATED')
  leaveFaction(@GamePerson('gamepersonId') gamepersonId) {
    return this.factionservice.leaveFaction(gamepersonId);
  }

  // used to change factions multiplier
  // not implemented in frontend uncomment this and services equivalent when needed
  // @Put('faction-multiplier/:id')
  // @Roles('admin')
  // @GameStates('STARTED')
  // factionMultiplier(@Param('id') game, @Body() body: FactionDTO) {
  //   return this.factionservice.changeFactionMultiplier(body);
  // }

  // check if person belongs to a faction in a game
  @Get('check-faction/:id')
  @UseGuards(new AuthGuard())
  checkFaction(@User('id') userId, @Param('id') gameId) {
    return this.factionservice.verifyUser(userId, gameId);
  }
}
